@@ -142,28 +142,6 @@ panel_applet_frame_draw (GtkWidget *widget,
         return FALSE;
 }
 
-static void
-panel_applet_frame_update_background_size (PanelAppletFrame *frame,
-					   GtkAllocation    *old_allocation,
-					   GtkAllocation    *new_allocation)
-{
-	PanelBackground *background;
-
-	if (old_allocation->x      == new_allocation->x &&
-	    old_allocation->y      == new_allocation->y &&
-	    old_allocation->width  == new_allocation->width &&
-	    old_allocation->height == new_allocation->height)
-		return;
-
-	background = &frame->priv->panel->background;
-
-	if (background->type == PANEL_BACK_NONE ||
-	   (background->type == PANEL_BACK_COLOR && !background->has_alpha))
-		return;
-
-	panel_applet_frame_change_background (frame, background->type);
-}
-
 static void
 panel_applet_frame_get_preferred_width(GtkWidget *widget, gint *minimal_width, gint *natural_width)
 {
@@ -242,6 +220,25 @@ panel_applet_frame_get_preferred_height(GtkWidget *widget, gint *minimal_height,
 	}
 }
 
+static void
+change_background (PanelAppletFrame *frame,
+                   GtkAllocation    *old,
+                   GtkAllocation    *new)
+{
+	PanelBackgroundType type;
+
+	if (old->x == new->x &&
+	    old->y == new->y &&
+	    old->width == new->width &&
+	    old->height == new->height)
+		return;
+
+	type = frame->priv->panel->background.type;
+
+	PANEL_APPLET_FRAME_GET_CLASS (frame)->change_background (frame,
+	                                                         type);
+}
+
 static void
 panel_applet_frame_size_allocate (GtkWidget     *widget,
 				  GtkAllocation *allocation)
@@ -251,7 +248,6 @@ panel_applet_frame_size_allocate (GtkWidget     *widget,
 	GtkWidget        *child;
 	GdkWindow        *window;
 	GtkAllocation     new_allocation;
-	GtkAllocation     old_allocation;
 	GtkAllocation     widget_allocation;
 
 	frame = PANEL_APPLET_FRAME (widget);
@@ -259,21 +255,15 @@ panel_applet_frame_size_allocate (GtkWidget     *widget,
 
 	gtk_widget_get_allocation (widget, &widget_allocation);
 
-	old_allocation.x      = widget_allocation.x;
-	old_allocation.y      = widget_allocation.y;
-	old_allocation.width  = widget_allocation.width;
-	old_allocation.height = widget_allocation.height;
-
 	if (!frame->priv->has_handle) {
 		GTK_WIDGET_CLASS (panel_applet_frame_parent_class)->size_allocate (widget,
 										   allocation);
-		panel_applet_frame_update_background_size (frame,
-							   &old_allocation,
-							   allocation);
+		change_background (frame, &widget_allocation, allocation);
 		return;
 	}
 
 	gtk_widget_set_allocation (widget, allocation);
+	change_background (frame, &widget_allocation, allocation);
 
 	frame->priv->handle_rect.x = 0;
 	frame->priv->handle_rect.y = 0;
@@ -343,10 +333,6 @@ panel_applet_frame_size_allocate (GtkWidget     *widget,
 		gtk_widget_size_allocate (child, &new_allocation);
   
 	frame->priv->child_allocation = new_allocation;
-
-	panel_applet_frame_update_background_size (frame,
-						   &old_allocation,
-						   allocation);
 }
 
 static inline gboolean
@@ -521,36 +507,6 @@ panel_applet_frame_change_orientation (PanelAppletFrame *frame,
 	PANEL_APPLET_FRAME_GET_CLASS (frame)->change_orientation (frame, orientation);
 }
 
-void
-panel_applet_frame_change_size (PanelAppletFrame *frame,
-				guint             size)
-{
-	PANEL_APPLET_FRAME_GET_CLASS (frame)->change_size (frame, size);
-}
-
-void
-panel_applet_frame_change_background (PanelAppletFrame    *frame,
-				      PanelBackgroundType  type)
-{
-	GtkWidget *parent;
-
-	g_return_if_fail (PANEL_IS_APPLET_FRAME (frame));
-
-	parent = gtk_widget_get_parent (GTK_WIDGET (frame));
-
-	g_return_if_fail (PANEL_IS_WIDGET (parent));
-
-	if (frame->priv->has_handle) {
-		PanelBackground *background;
-
-		background = &PANEL_WIDGET (parent)->background;
-		panel_background_change_background_on_widget (background,
-							      GTK_WIDGET (frame));
-	}
-
-	PANEL_APPLET_FRAME_GET_CLASS (frame)->change_background (frame, type);
-}
-
 void
 panel_applet_frame_set_panel (PanelAppletFrame *frame,
 			      PanelWidget      *panel)
@@ -625,22 +581,10 @@ _panel_applet_frame_update_flags (PanelAppletFrame *frame,
 				  gboolean          minor,
 				  gboolean          has_handle)
 {
-	gboolean old_has_handle;
-
 	panel_widget_set_applet_expandable (
 		frame->priv->panel, GTK_WIDGET (frame), major, minor);
 
-	old_has_handle = frame->priv->has_handle;
 	frame->priv->has_handle = has_handle;
-
-	if (!old_has_handle && frame->priv->has_handle) {
-		/* we've added an handle, so we need to get the background for
-		 * it */
-		PanelBackground *background;
-
-		background = &frame->priv->panel->background;
-		panel_applet_frame_change_background (frame, background->type);
-	}
 }
 
 void
@@ -865,12 +809,6 @@ panel_applet_frame_activating_get_orientation (PanelAppletFrameActivating *frame
 	return panel_widget_get_applet_orientation (frame_act->panel);
 }
 
-guint32
-panel_applet_frame_activating_get_size (PanelAppletFrameActivating *frame_act)
-{
-	return frame_act->panel->sz;
-}
-
 gboolean
 panel_applet_frame_activating_get_locked_down (PanelAppletFrameActivating *frame_act)
 {
@@ -891,12 +829,6 @@ panel_applet_frame_activating_get_settings_path (PanelAppletFrameActivating *fra
 	return path_instance;
 }
 
-gchar *
-panel_applet_frame_activating_get_conf_path (PanelAppletFrameActivating *frame_act)
-{
-	return panel_layout_object_get_gconf_path (frame_act->id);
-}
-
 static void
 panel_applet_frame_loading_failed_response (GtkWidget *dialog,
 					    guint      response,
@@ -949,15 +881,15 @@ panel_applet_frame_loading_failed (const char  *iid,
 
 	if (locked_down) {
 		gtk_dialog_add_buttons (GTK_DIALOG (dialog),
-					GTK_STOCK_OK, LOADING_FAILED_RESPONSE_DONT_DELETE,
+					_("OK"), LOADING_FAILED_RESPONSE_DONT_DELETE,
 					NULL);
 	} else {
 		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
 					_("Do you want to delete the applet "
 					  "from your configuration?"));
 		gtk_dialog_add_buttons (GTK_DIALOG (dialog),
-					PANEL_STOCK_DONT_DELETE, LOADING_FAILED_RESPONSE_DONT_DELETE,
-					GTK_STOCK_DELETE, LOADING_FAILED_RESPONSE_DELETE,
+					_("D_on't Delete"), LOADING_FAILED_RESPONSE_DONT_DELETE,
+					_("_Delete"), LOADING_FAILED_RESPONSE_DELETE,
 					NULL);
 	}
 
