@@ -996,129 +996,3 @@ panel_background_effective_type (PanelBackground *background)
 
 	return retval;
 }
-
-static cairo_pattern_t *
-panel_background_get_pattern_for_widget (PanelBackground *background,
-                                         GtkWidget       *widget)
-{
-	GtkAllocation    allocation;
-	cairo_t         *cr;
-        cairo_surface_t *surface;
-        cairo_surface_t *bg_surface;
-	cairo_pattern_t *pattern;
-        cairo_matrix_t   matrix;
-
-        if (!background->composited_pattern)
-                return NULL;
-
-        if (cairo_pattern_get_surface (background->composited_pattern, &bg_surface) != CAIRO_STATUS_SUCCESS)
-                return NULL;
-
-	gtk_widget_get_allocation (widget, &allocation);
-        surface = cairo_image_surface_create (CAIRO_FORMAT_RGB24,
-                                              allocation.width, allocation.height);
-
-	cr = cairo_create (surface);
-        cairo_set_source_surface (cr, bg_surface, -allocation.x, -allocation.y);
-        cairo_rectangle (cr, 0, 0, allocation.width, allocation.height);
-        cairo_fill (cr);
-        cairo_destroy (cr);
-
-        pattern = cairo_pattern_create_for_surface (surface);
-        cairo_matrix_init_translate (&matrix, 0, 0);
-        cairo_matrix_scale (&matrix, allocation.width, allocation.height);
-        cairo_pattern_set_matrix (pattern, &matrix);
-        cairo_pattern_set_extend (pattern, CAIRO_EXTEND_PAD);
-
-        cairo_surface_destroy (surface);
-
-        return pattern;
-}
-
-static GtkStyleProperties *
-_panel_background_get_widget_style_properties (GtkWidget *widget,
-                                               gboolean   create_if_needed)
-{
-        GtkStyleProperties *properties;
-
-        properties = g_object_get_data (G_OBJECT (widget),
-                                        "panel-object-style-props");
-
-        if (!properties && create_if_needed) {
-                properties = gtk_style_properties_new ();
-                g_object_set_data_full (G_OBJECT (widget),
-                                        "panel-object-style-props",
-                                        properties,
-                                        (GDestroyNotify) g_object_unref);
-        }
-
-        return properties;
-}
-
-static void
-_panel_background_reset_widget_style_properties (GtkWidget *widget)
-{
-        GtkStyleProperties *properties;
-
-        properties = _panel_background_get_widget_style_properties (widget, FALSE);
-
-        if (properties)
-                gtk_style_context_remove_provider (gtk_widget_get_style_context (widget),
-                                                   GTK_STYLE_PROVIDER (properties));
-
-
-        g_object_set_data (G_OBJECT (widget), "panel-object-style-props", NULL);
-}
-
-void
-panel_background_change_background_on_widget (PanelBackground *background,
-					      GtkWidget       *widget)
-{
-        GtkStyleProperties *properties;
-
-        gtk_widget_reset_style (widget);
-
-	switch (panel_background_get_type (background)) {
-	case PANEL_BACK_NONE:
-                _panel_background_reset_widget_style_properties (widget);
-                return;
-	case PANEL_BACK_COLOR:
-                if (!background->has_alpha) {
-                        properties = _panel_background_get_widget_style_properties (widget, TRUE);
-                        gtk_style_properties_set (properties, GTK_STATE_FLAG_NORMAL,
-                                                  "background-color", &background->color,
-                                                  "background-image", NULL,
-                                                  NULL);
-                        break;
-                }
-                // Color with alpha, fallback to image
-	case PANEL_BACK_IMAGE: {
-                cairo_pattern_t *pattern;
-
-                properties = _panel_background_get_widget_style_properties (widget, TRUE);
-                pattern = panel_background_get_pattern_for_widget (background, widget);
-                if (pattern) {
-                        gtk_style_properties_set (properties, GTK_STATE_FLAG_NORMAL,
-                                                  /* background-color can't be
-                                                   * NULL, but is ignored
-                                                   * anyway */
-                                                  "background-image", pattern,
-                                                  NULL);
-                        cairo_pattern_destroy (pattern);
-                } else {
-                        _panel_background_reset_widget_style_properties (widget);
-                        return;
-                }
-        }
-		break;
-	default:
-		g_assert_not_reached ();
-		break;
-	}
-
-	/* Note: this actually replaces the old properties, since it's the same
-	 * pointer */
-        gtk_style_context_add_provider (gtk_widget_get_style_context (widget),
-                                        GTK_STYLE_PROVIDER (properties),
-                                        GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
-}
