@@ -1536,24 +1536,26 @@ panel_widget_set_background_default_style (GtkWidget *widget)
 {
 	GtkStyleContext *context;
 	GtkStateFlags    state;
-        GdkRGBA          bg_color;
-        cairo_pattern_t *bg_image;
+	GdkRGBA         *bg_color;
+	cairo_pattern_t *bg_image;
+	PanelBackground *background;
 
 	if (gtk_widget_get_realized (widget)) {
 		context = gtk_widget_get_style_context (widget);
 		state = gtk_widget_get_state_flags (widget);
+		background = &PANEL_WIDGET (widget)->background;
 
-                gtk_style_context_get_background_color (context, state, &bg_color);
-                gtk_style_context_get (context, state,
-                                       "background-image", &bg_image,
-                                       NULL);
+		gtk_style_context_get (context, state,
+		                       "background-color", &bg_color,
+		                       "background-image", &bg_image,
+		                       NULL);
 
-		panel_background_set_default_style (
-			&PANEL_WIDGET (widget)->background,
-			&bg_color, bg_image);
+		panel_background_set_default_style (background,
+		                                    bg_color,
+		                                    bg_image);
 
-                if (bg_image)
-                        cairo_pattern_destroy (bg_image);
+		if (bg_image)
+			cairo_pattern_destroy (bg_image);
 	}
 }
 
@@ -1689,7 +1691,7 @@ panel_widget_init (PanelWidget *panel)
 	
 	panel->packed        = FALSE;
 	panel->orient        = GTK_ORIENTATION_HORIZONTAL;
-	panel->size          = G_MAXINT;
+	panel->size          = 0;
 	panel->applet_list   = NULL;
 	panel->drop_widget   = widget;
 	panel->open_dialogs  = NULL;
@@ -1718,21 +1720,18 @@ panel_widget_new (PanelToplevel  *toplevel,
 
 	panel = g_object_new (PANEL_TYPE_WIDGET, NULL);
 
-        gtk_widget_set_has_window (GTK_WIDGET (panel), TRUE);
-        gtk_widget_set_can_focus (GTK_WIDGET (panel), TRUE);
+	gtk_widget_set_has_window (GTK_WIDGET (panel), TRUE);
+	gtk_widget_set_can_focus (GTK_WIDGET (panel), TRUE);
 
 	panel->orient = orient;
 	panel->sz = sz;
 
 	panel->packed = packed;
-	if (packed)
-		panel->size = 0;
-	else
-		panel->size = G_MAXINT;
+	panel->size = 0;
 
 	panel->toplevel    = toplevel;
 	panel->drop_widget = GTK_WIDGET (toplevel);
-	
+
 	return GTK_WIDGET (panel);
 }
 
@@ -1839,7 +1838,8 @@ panel_widget_applet_drag_start (PanelWidget *panel,
 		GdkDevice     *pointer;
 		GdkDeviceManager *device_manager;
 
-		fleur_cursor = gdk_cursor_new (GDK_FLEUR);
+		fleur_cursor = gdk_cursor_new_for_display (gdk_display_get_default (),
+		                                           GDK_FLEUR);
 
 		display = gdk_window_get_display (window);
 		device_manager = gdk_display_get_device_manager (display);
@@ -2490,7 +2490,8 @@ panel_widget_reparent (PanelWidget         *old_panel,
 	panel_widget_reset_saved_focus (old_panel);
 	if (gtk_container_get_focus_child (GTK_CONTAINER (old_panel)) == applet)
 		focus_widget = gtk_window_get_focus (GTK_WINDOW (old_panel->toplevel));
-	gtk_widget_reparent (applet, GTK_WIDGET (new_panel));
+	gtk_container_remove (GTK_CONTAINER (old_panel), applet);
+	gtk_container_add (GTK_CONTAINER (new_panel), applet);
 
 	if (info && info->type == PANEL_OBJECT_APPLET)
 		panel_applet_frame_set_panel (PANEL_APPLET_FRAME (ad->applet), new_panel);
