@@ -36,12 +36,11 @@
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 #include <gtk/gtkx.h>
-#include <gconf/gconf.h>
-#include <gconf/gconf-client.h>
 #include <X11/Xatom.h>
 #include <cairo-xlib.h>
 
 #include "panel-applet.h"
+#include "panel-applet-private.h"
 #include "panel-applet-bindings.h"
 #include "panel-applet-factory.h"
 #include "panel-applet-marshal.h"
@@ -75,28 +74,25 @@
  * your binary.
  */
 
-#define PANEL_APPLET_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PANEL_TYPE_APPLET, PanelAppletPrivate))
-
 struct _PanelAppletPrivate {
 	GtkWidget         *plug;
 	GtkWidget         *applet;
-	GConfClient       *client;
 	GDBusConnection   *connection;
 
+	gboolean           out_of_process;
+
 	char              *id;
 	GClosure          *closure;
 	char              *object_path;
 	guint              object_id;
 	char              *settings_path;
-	char              *prefs_key;
 
-	GtkUIManager      *ui_manager;
-	GtkActionGroup    *applet_action_group;
-	GtkActionGroup    *panel_action_group;
+	GtkBuilder         *builder;
+	GSimpleActionGroup *applet_action_group;
+	GSimpleActionGroup *panel_action_group;
 
 	PanelAppletFlags   flags;
 	PanelAppletOrient  orient;
-	guint              size;
 	char              *background;
 	GtkWidget         *background_widget;
 
@@ -113,7 +109,6 @@ struct _PanelAppletPrivate {
 
 enum {
         CHANGE_ORIENT,
-        CHANGE_SIZE,
         CHANGE_BACKGROUND,
 	MOVE_FOCUS_OUT_OF_APPLET,
         LAST_SIGNAL
@@ -123,13 +118,12 @@ static guint panel_applet_signals [LAST_SIGNAL];
 
 enum {
 	PROP_0,
+	PROP_OUT_OF_PROCESS,
 	PROP_ID,
 	PROP_CLOSURE,
 	PROP_CONNECTION,
 	PROP_SETTINGS_PATH,
-	PROP_PREFS_KEY,
 	PROP_ORIENT,
-	PROP_SIZE,
 	PROP_BACKGROUND,
 	PROP_FLAGS,
 	PROP_SIZE_HINTS,
@@ -137,241 +131,45 @@ enum {
 };
 
 static void       panel_applet_handle_background   (PanelApplet       *applet);
-static GtkAction *panel_applet_menu_get_action     (PanelApplet       *applet,
+static GAction   *panel_applet_menu_get_action     (PanelApplet       *applet,
 						    const gchar       *action);
 static void       panel_applet_menu_update_actions (PanelApplet       *applet);
-static void       panel_applet_menu_cmd_remove     (GtkAction         *action,
-						    PanelApplet       *applet);
-static void       panel_applet_menu_cmd_move       (GtkAction         *action,
-						    PanelApplet       *applet);
+static void       panel_applet_menu_cmd_remove     (GSimpleAction     *action,
+						    GVariant      *parameter,
+						    gpointer       user_data);
+static void       panel_applet_menu_cmd_move       (GSimpleAction     *action,
+						    GVariant      *parameter,
+						    gpointer       user_data);
 static void       panel_applet_register_object     (PanelApplet       *applet);
 
 static const gchar panel_menu_ui[] =
-	"<ui>\n"
-	"  <popup name=\"PanelAppletPopup\" action=\"PopupAction\">\n"
-	"    <placeholder name=\"AppletItems\"/>\n"
-	"  </popup>\n"
-	"  <popup name=\"PanelAppletEditPopup\" action=\"PopupEditAction\">\n"
-	"    <menuitem name=\"MoveItem\" action=\"Move\"/>\n"
-	"    <menuitem name=\"RemoveItem\" action=\"Remove\"/>\n"
-	"  </popup>\n"
-	"</ui>\n";
+	"<interface>\n"
+	" <menu id=\"panel-applet-popup\">"
+	" </menu>"
+	" <menu id=\"panel-applet-edit-popup\">"
+	"   <section>"
+	"     <item>"
+	"       <attribute name=\"label\" translatable=\"no\">%s</attribute>"
+	"       <attribute name=\"action\">libpanel-applet.remove</attribute>"
+	"     </item>"
+	"     <item>"
+	"       <attribute name=\"label\" translatable=\"no\">%s</attribute>"
+	"       <attribute name=\"action\">libpanel-applet.move</attribute>"
+	"     </item>"
+	"   </section>"
+	" </menu>"
+	"</interface>\n";
 
-static const GtkActionEntry menu_entries[] = {
-	{ "PopupAction", NULL, "Popup Action",
-	  NULL, NULL,
-	  NULL },
-	{ "Remove", GTK_STOCK_REMOVE, N_("_Remove From Panel"),
-	  NULL, NULL,
-	  G_CALLBACK (panel_applet_menu_cmd_remove) },
-	{ "Move", NULL, N_("_Move"),
-	  NULL, NULL,
-	  G_CALLBACK (panel_applet_menu_cmd_move) }
+static const GActionEntry menu_entries[] = {
+	{ "remove", panel_applet_menu_cmd_remove, NULL, NULL, NULL },
+	{ "move",   panel_applet_menu_cmd_move,   NULL, NULL, NULL }
 };
 
-G_DEFINE_TYPE (PanelApplet, panel_applet, GTK_TYPE_EVENT_BOX)
+G_DEFINE_TYPE_WITH_PRIVATE (PanelApplet, panel_applet, GTK_TYPE_EVENT_BOX)
 
 #define PANEL_APPLET_INTERFACE   "org.gnome.panel.applet.Applet"
 #define PANEL_APPLET_OBJECT_PATH "/org/gnome/panel/applet/%s/%d"
 
-static void
-panel_applet_associate_schemas_in_dir (GConfClient  *client,
-				       const gchar  *prefs_key,
-				       const gchar  *schema_dir,
-				       GError      **error)
-{
-	GSList *list, *l;
-
-	g_assert (error != NULL);
-
-	list = gconf_client_all_entries (client, schema_dir, error);
-
-	if (*error != NULL)
-		return;
-
-	for (l = list; l; l = l->next) {
-		GConfEntry  *entry = l->data;
-		const gchar *schema_key;
-		GConfEntry  *applet_entry;
-		const gchar *applet_schema_key;
-		gchar       *key;
-		gchar       *tmp;
-
-		schema_key = gconf_entry_get_key (entry);
-		tmp = g_path_get_basename (schema_key);
-
-		if (strchr (tmp, '-'))
-			g_warning ("Applet key '%s' contains a hyphen. Please "
-				   "use underscores in gconf keys\n", tmp);
-
-		key = g_strdup_printf ("%s/%s", prefs_key, tmp);
-		g_free (tmp);
-
-		/* Associating a schema is potentially expensive, so let's try
-		 * to avoid this by doing it only when needed. So we check if
-		 * the key is already correctly associated. */
-
-		applet_entry = gconf_client_get_entry (client, key,
-						       NULL, TRUE, NULL);
-		if (applet_entry)
-			applet_schema_key = gconf_entry_get_schema_name (applet_entry);
-		else
-			applet_schema_key = NULL;
-
-		if (g_strcmp0 (schema_key, applet_schema_key) != 0) {
-			gconf_engine_associate_schema (client->engine,
-						       key, schema_key, error);
-
-			if (applet_entry == NULL ||
-			    gconf_entry_get_value (applet_entry) == NULL ||
-			    gconf_entry_get_is_default (applet_entry)) {
-				/* unset the key: gconf_client_get_entry()
-				 * brought an invalid entry in the client
-				 * cache, and we want to fix this */
-				gconf_client_unset (client, key, NULL);
-			}
-		}
-
-		g_free (key);
-
-		if (applet_entry)
-			gconf_entry_unref (applet_entry);
-		gconf_entry_unref (entry);
-
-		if (*error) {
-			g_slist_free (list);
-			return;
-		}
-	}
-
-	g_slist_free (list);
-
-	list = gconf_client_all_dirs (client, schema_dir, error);
-
-	for (l = list; l; l = l->next) {
-		gchar *subdir = l->data;
-		gchar *prefs_subdir;
-		gchar *schema_subdir;
-		gchar *tmp;
-
-		tmp = g_path_get_basename (subdir);
-
-		prefs_subdir  = g_strdup_printf ("%s/%s", prefs_key, tmp);
-		schema_subdir = g_strdup_printf ("%s/%s", schema_dir, tmp);
-
-		panel_applet_associate_schemas_in_dir (client,
-						       prefs_subdir,
-						       schema_subdir,
-						       error);
-
-		g_free (prefs_subdir);
-		g_free (schema_subdir);
-		g_free (subdir);
-		g_free (tmp);
-
-		if (*error) {
-			g_slist_free (list);
-			return;
-		}
-	}
-
-	g_slist_free (list);
-}
-
-/**
- * panel_applet_add_preferences:
- * @applet: a #PanelApplet.
- * @schema_dir: a GConf path of a schema directory.
- * @error: a #GError, or %NULL.
- *
- * Associates the per-instance GConf directory of @applet to the schemas
- * defined in @schema_dir. This must be called if the applet will save
- * per-instance settings, to ensure that each key in the per-instance GConf
- * directory has a defined type, sane default and documentation.
- *
- * Deprecated: 3.0: Use #GSettings to store per-instance settings.
- **/
-void
-panel_applet_add_preferences (PanelApplet  *applet,
-			      const gchar  *schema_dir,
-			      GError      **error)
-{
-	GError **_error = NULL;
-	GError  *our_error = NULL;
-
-	g_return_if_fail (PANEL_IS_APPLET (applet));
-	g_return_if_fail (schema_dir != NULL);
-
-	if (!applet->priv->prefs_key)
-		return;
-
-	/* panel_applet_associate_schemas_in_dir() requires a non-NULL error */
-	if (error)
-		_error = error;
-	else
-		_error = &our_error;
-
-	panel_applet_associate_schemas_in_dir (applet->priv->client,
-					       applet->priv->prefs_key,
-					       schema_dir,
-					       _error);
-
-	if (!error && our_error)
-		g_error_free (our_error);
-}
-
-/**
- * panel_applet_get_preferences_key:
- * @applet: a #PanelApplet.
- *
- * Gets the GConf path to the per-instance settings of @applet.
- *
- * Returns: a copy of the GConf path to the per-instance settings of @applet.
- *
- * Deprecated: 3.0: Use #GSettings to store per-instance settings.
- **/
-char *
-panel_applet_get_preferences_key (PanelApplet *applet)
-{
-	g_return_val_if_fail (PANEL_IS_APPLET (applet), NULL);
-
-	if (!applet->priv->prefs_key)
-		return NULL;
-
-	return g_strdup (applet->priv->prefs_key);
-}
-
-static void
-panel_applet_set_preferences_key (PanelApplet *applet,
-				  const char  *prefs_key)
-{
-	if (applet->priv->prefs_key == prefs_key)
-		return;
-
-	if (g_strcmp0 (applet->priv->prefs_key, prefs_key) == 0)
-		return;
-
-	if (applet->priv->prefs_key) {
-		gconf_client_remove_dir (applet->priv->client,
-					 applet->priv->prefs_key,
-					 NULL);
-
-		g_free (applet->priv->prefs_key);
-		applet->priv->prefs_key = NULL;
-	}
-
-	if (prefs_key) {
-		applet->priv->prefs_key = g_strdup (prefs_key);
-
-		gconf_client_add_dir (applet->priv->client,
-				      applet->priv->prefs_key,
-				      GCONF_CLIENT_PRELOAD_RECURSIVE,
-				      NULL);
-	}
-
-	g_object_notify (G_OBJECT (applet), "prefs-key");
-}
-
 /**
  * panel_applet_settings_new:
  * @applet: a #PanelApplet.
@@ -380,7 +178,7 @@ panel_applet_set_preferences_key (PanelApplet *applet,
  * Creates a new #GSettings object for the per-instance settings of @applet,
  * with a given schema.
  *
- * Returns: a new #GSettings object for the per-instance settings of @applet.
+ * Returns: (transfer full): a new #GSettings object for the per-instance settings of @applet.
  **/
 GSettings *
 panel_applet_settings_new (PanelApplet *applet,
@@ -590,44 +388,6 @@ panel_applet_set_size_hints (PanelApplet *applet,
 	}
 }
 
-/**
- * panel_applet_get_size:
- * @applet: a #PanelApplet.
- *
- * Gets the size of the panel @applet is on. For a horizontal panel, the
- * size if the height of the panel; for a vertical panel, the size is the width
- * of the panel.
- *
- * Returns: the size of the panel @applet is on.
- *
- * Deprecated: 3.0: Use the allocation of @applet instead.
- **/
-guint
-panel_applet_get_size (PanelApplet *applet)
-{
-	g_return_val_if_fail (PANEL_IS_APPLET (applet), 0);
-
-	return applet->priv->size;
-}
-
-/* Applets cannot set their size, so API is not public. */
-static void
-panel_applet_set_size (PanelApplet *applet,
-		       guint        size)
-{
-	g_return_if_fail (PANEL_IS_APPLET (applet));
-
-	if (applet->priv->size == size)
-		return;
-
-	applet->priv->size = size;
-	g_signal_emit (G_OBJECT (applet),
-		       panel_applet_signals [CHANGE_SIZE],
-		       0, size);
-
-	g_object_notify (G_OBJECT (applet), "size");
-}
-
 /**
  * panel_applet_get_orient:
  * @applet: a #PanelApplet.
@@ -860,30 +620,37 @@ panel_applet_request_focus (PanelApplet	 *applet,
 		    &xev);
 }
 
-static GtkAction *
+static GAction *
 panel_applet_menu_get_action (PanelApplet *applet,
 			      const gchar *action)
 {
-	return gtk_action_group_get_action (applet->priv->panel_action_group, action);
+	return g_action_map_lookup_action (G_ACTION_MAP (applet->priv->panel_action_group),
+	                                   action);
 }
 
 static void
 panel_applet_menu_update_actions (PanelApplet *applet)
 {
-	gboolean locked_down = applet->priv->locked_down;
+	gboolean locked_down;
+	GAction *action;
 
-	g_object_set (panel_applet_menu_get_action (applet, "Move"),
-		      "visible", !locked_down,
-		      NULL);
-	g_object_set (panel_applet_menu_get_action (applet, "Remove"),
-		      "visible", !locked_down,
-		      NULL);
+	locked_down = applet->priv->locked_down;
+
+	action = panel_applet_menu_get_action (applet, "Move");
+	if (action)
+		g_object_set (action, "enabled", !locked_down, NULL);
+
+	action = panel_applet_menu_get_action (applet, "Remove");
+	if (action)
+		g_object_set (action, "enabled", !locked_down, NULL);
 }
 
 static void
-panel_applet_menu_cmd_remove (GtkAction   *action,
-			      PanelApplet *applet)
+panel_applet_menu_cmd_remove (GSimpleAction *action,
+			      GVariant      *parameter,
+			      gpointer       user_data)
 {
+	PanelApplet *applet = PANEL_APPLET (user_data);
 	GError *error = NULL;
 
 	if (!applet->priv->connection)
@@ -903,9 +670,11 @@ panel_applet_menu_cmd_remove (GtkAction   *action,
 }
 
 static void
-panel_applet_menu_cmd_move (GtkAction   *action,
-			    PanelApplet *applet)
+panel_applet_menu_cmd_move (GSimpleAction *action,
+			    GVariant      *parameter,
+			    gpointer       user_data)
 {
+	PanelApplet *applet = PANEL_APPLET (user_data);
 	GError *error = NULL;
 
 	if (!applet->priv->connection)
@@ -928,11 +697,12 @@ panel_applet_menu_cmd_move (GtkAction   *action,
  * panel_applet_setup_menu:
  * @applet: a #PanelApplet.
  * @xml: a menu XML string.
- * @action_group: a #GtkActionGroup.
+ * @action_group: a #GSimpleActionGroup.
+ * @translation_domain: gettext translation domain
  *
  * Sets up the context menu of @applet. @xml is a #GtkUIManager UI definition,
  * describing how to display the menu items. @action_group contains the
- * various #GtkAction that are referenced in @xml.
+ * various #GSimpleAction that are referenced in @xml.
  *
  * See also the <link linkend="getting-started.context-menu">Context
  * Menu</link> section.
@@ -940,7 +710,8 @@ panel_applet_menu_cmd_move (GtkAction   *action,
 void
 panel_applet_setup_menu (PanelApplet    *applet,
 			 const gchar    *xml,
-			 GtkActionGroup *action_group)
+			 GSimpleActionGroup *action_group,
+			 const gchar        *translation_domain)
 {
 	gchar  *new_xml;
 	GError *error = NULL;
@@ -951,16 +722,14 @@ panel_applet_setup_menu (PanelApplet    *applet,
 	if (applet->priv->applet_action_group)
 		return;
 
-	applet->priv->applet_action_group = g_object_ref (action_group);
-	gtk_ui_manager_insert_action_group (applet->priv->ui_manager,
-					    action_group, 0);
+	gtk_builder_set_translation_domain (applet->priv->builder, translation_domain);
 
-	new_xml = g_strdup_printf ("<ui><popup name=\"PanelAppletPopup\" action=\"AppletItems\">"
-				   "<placeholder name=\"AppletItems\">%s\n</placeholder>\n"
-				   "</popup></ui>\n", xml);
-	gtk_ui_manager_add_ui_from_string (applet->priv->ui_manager, new_xml, -1, &error);
+	applet->priv->applet_action_group = g_object_ref (action_group);
+
+	new_xml = g_strdup_printf ("<interface><menu id=\"panel-applet-popup\">%s</menu></interface>\n", xml);
+	gtk_builder_add_from_string (applet->priv->builder, new_xml, -1, &error);
 	g_free (new_xml);
-	gtk_ui_manager_ensure_update (applet->priv->ui_manager);
+
 	if (error) {
 		g_warning ("Error merging menus: %s\n", error->message);
 		g_error_free (error);
@@ -971,11 +740,12 @@ panel_applet_setup_menu (PanelApplet    *applet,
  * panel_applet_setup_menu_from_file:
  * @applet: a #PanelApplet.
  * @filename: path to a menu XML file.
- * @action_group: a #GtkActionGroup.
+ * @action_group: a #GSimpleActionGroup.
+ * @translation_domain: gettext translation domain
  *
  * Sets up the context menu of @applet. @filename is the path to a menu XML
  * file, containing a #GtkUIManager UI definition that describes how to display
- * the menu items. @action_group contains the various #GtkAction that
+ * the menu items. @action_group contains the various #GSimpleAction that
  * are referenced in @xml.
  *
  * See also the <link linkend="getting-started.context-menu">Context
@@ -984,13 +754,14 @@ panel_applet_setup_menu (PanelApplet    *applet,
 void
 panel_applet_setup_menu_from_file (PanelApplet    *applet,
 				   const gchar    *filename,
-				   GtkActionGroup *action_group)
+				   GSimpleActionGroup *action_group,
+				   const gchar        *translation_domain)
 {
 	gchar  *xml = NULL;
 	GError *error = NULL;
 
 	if (g_file_get_contents (filename, &xml, NULL, &error)) {
-		panel_applet_setup_menu (applet, xml, action_group);
+		panel_applet_setup_menu (applet, xml, action_group, translation_domain);
 	} else {
 		g_warning ("%s", error->message);
 		g_error_free (error);
@@ -1003,11 +774,12 @@ panel_applet_setup_menu_from_file (PanelApplet    *applet,
  * panel_applet_setup_menu_from_resource:
  * @applet: a #PanelApplet.
  * @resource_path: a resource path
- * @action_group: a #GtkActionGroup.
+ * @action_group: a #GSimpleActionGroup.
+ * @translation_domain: gettext translation domain
  *
  * Sets up the context menu of @applet. @filename is a resource path to a menu
  * XML file, containing a #GtkUIManager UI definition that describes how to
- * display the menu items. @action_group contains the various #GtkAction that
+ * display the menu items. @action_group contains the various #GSimpleAction that
  * are referenced in @xml.
  *
  * See also the <link linkend="getting-started.context-menu">Context
@@ -1018,7 +790,8 @@ panel_applet_setup_menu_from_file (PanelApplet    *applet,
 void
 panel_applet_setup_menu_from_resource (PanelApplet    *applet,
 				       const gchar    *resource_path,
-				       GtkActionGroup *action_group)
+				       GSimpleActionGroup *action_group,
+				       const gchar        *translation_domain)
 {
 	GBytes *bytes;
 	GError *error = NULL;
@@ -1030,7 +803,8 @@ panel_applet_setup_menu_from_resource (PanelApplet    *applet,
 	if (bytes) {
 		panel_applet_setup_menu (applet,
 					 g_bytes_get_data (bytes, NULL),
-					 action_group);
+					 action_group,
+					 translation_domain);
 	} else {
 		g_warning ("%s", error->message);
 		g_error_free (error);
@@ -1058,14 +832,6 @@ panel_applet_finalize (GObject *object)
 		applet->priv->object_path = NULL;
 	}
 
-	panel_applet_set_preferences_key (applet, NULL);
-
-	panel_applet_bindings_clean (applet->priv->client);
-
-	if (applet->priv->client)
-		g_object_unref (applet->priv->client);
-	applet->priv->client = NULL;
-
 	if (applet->priv->applet_action_group) {
 		g_object_unref (applet->priv->applet_action_group);
 		applet->priv->applet_action_group = NULL;
@@ -1076,13 +842,12 @@ panel_applet_finalize (GObject *object)
 		applet->priv->panel_action_group = NULL;
 	}
 
-	if (applet->priv->ui_manager) {
-		g_object_unref (applet->priv->ui_manager);
-		applet->priv->ui_manager = NULL;
+	if (applet->priv->builder) {
+		g_object_unref (applet->priv->builder);
+		applet->priv->builder = NULL;
 	}
 
 	g_free (applet->priv->size_hints);
-	g_free (applet->priv->prefs_key);
 	g_free (applet->priv->settings_path);
 	g_free (applet->priv->background);
 	g_free (applet->priv->id);
@@ -1199,11 +964,14 @@ panel_applet_menu_popup (PanelApplet *applet,
 			 guint32      time)
 {
 	GtkWidget *menu;
+	GMenu     *gmenu;
 	GList     *children, *l;
 	gboolean   visible = FALSE;
 
-	menu = gtk_ui_manager_get_widget (applet->priv->ui_manager,
-					  "/PanelAppletPopup");
+	gmenu = G_MENU (gtk_builder_get_object (applet->priv->builder, "panel-applet-popup"));
+	menu = gtk_menu_new_from_model (G_MENU_MODEL (gmenu));
+
+	gtk_menu_attach_to_widget (GTK_MENU (menu), GTK_WIDGET (applet), NULL);
 
 	children = gtk_container_get_children (GTK_CONTAINER (menu));
 	for (l = children; l != NULL; l = l->next) {
@@ -1229,12 +997,16 @@ panel_applet_edit_menu_popup (PanelApplet *applet,
 			      guint32      time)
 {
 	GtkWidget *menu;
+	GMenu     *gmenu;
 
 	if (applet->priv->locked_down)
 		return;
 
-	menu = gtk_ui_manager_get_widget (applet->priv->ui_manager,
-					  "/PanelAppletEditPopup");
+	gmenu = G_MENU (gtk_builder_get_object (applet->priv->builder, "panel-applet-edit-popup"));
+	menu = gtk_menu_new_from_model (G_MENU_MODEL (gmenu));
+
+	gtk_menu_attach_to_widget (GTK_MENU (menu), GTK_WIDGET (applet), NULL);
+
 	gtk_menu_set_screen (GTK_MENU (menu), gtk_widget_get_screen (GTK_WIDGET (applet)));
 	gtk_menu_popup (GTK_MENU (menu),
 			NULL, NULL,
@@ -1261,13 +1033,19 @@ panel_applet_can_focus (GtkWidget *widget)
 
 /* Taken from libbonoboui/bonobo/bonobo-plug.c */
 static gboolean
-panel_applet_button_event (GtkWidget      *widget,
-			   GdkEventButton *event)
+panel_applet_button_event (PanelApplet    *applet,
+                           GdkEventButton *event)
 {
+	GtkWidget *widget;
 	GdkWindow *window;
 	GdkWindow *socket_window;
 	XEvent     xevent;
 
+	if (!applet->priv->out_of_process)
+		return FALSE;
+
+	widget = applet->priv->plug;
+
 	if (!gtk_widget_is_toplevel (widget))
 		return FALSE;
 
@@ -1340,7 +1118,7 @@ panel_applet_button_press (GtkWidget      *widget,
 		return TRUE;
 	}
 
-	return panel_applet_button_event (applet->priv->plug, event);
+	return panel_applet_button_event (applet, event);
 }
 
 static gboolean
@@ -1349,7 +1127,7 @@ panel_applet_button_release (GtkWidget      *widget,
 {
 	PanelApplet *applet = PANEL_APPLET (widget);
 
-	return panel_applet_button_event (applet->priv->plug, event);
+	return panel_applet_button_event (applet, event);
 }
 
 static gboolean
@@ -1612,7 +1390,6 @@ panel_applet_get_pattern_from_pixmap (PanelApplet *applet,
         cairo_t         *cr;
         cairo_surface_t *background;
         cairo_surface_t *surface;
-        cairo_matrix_t   matrix;
         cairo_pattern_t *pattern;
 
 	g_return_val_if_fail (PANEL_IS_APPLET (applet), NULL);
@@ -1652,13 +1429,8 @@ panel_applet_get_pattern_from_pixmap (PanelApplet *applet,
 
 	pattern = NULL;
 
-        if (cairo_status (cr) == CAIRO_STATUS_SUCCESS) {
-                pattern = cairo_pattern_create_for_surface (surface);
-                cairo_matrix_init_translate (&matrix, 0, 0);
-                cairo_matrix_scale (&matrix, width, height);
-                cairo_pattern_set_matrix (pattern, &matrix);
-                cairo_pattern_set_extend (pattern, CAIRO_EXTEND_PAD);
-        }
+	if (cairo_status (cr) == CAIRO_STATUS_SUCCESS)
+		pattern = cairo_pattern_create_for_surface (surface);
 
         cairo_destroy (cr);
         cairo_surface_destroy (surface);
@@ -1742,82 +1514,19 @@ panel_applet_set_background_string (PanelApplet *applet,
 	g_object_notify (G_OBJECT (applet), "background");
 }
 
-static GtkStyleProperties *
-_panel_applet_get_widget_style_properties (GtkWidget *widget,
-                                           gboolean   create_if_needed)
-{
-        GtkStyleProperties *properties;
-
-        properties = g_object_get_data (G_OBJECT (widget),
-                                        "panel-applet-style-props");
-
-        if (!properties && create_if_needed) {
-                properties = gtk_style_properties_new ();
-                g_object_set_data_full (G_OBJECT (widget),
-                                        "panel-applet-style-props",
-                                        properties,
-                                        (GDestroyNotify) g_object_unref);
-        }
-
-        return properties;
-}
-
-static void
-_panel_applet_reset_widget_style_properties (GtkWidget *widget)
-{
-        GtkStyleProperties *properties;
-
-        properties = _panel_applet_get_widget_style_properties (widget, FALSE);
-
-        if (properties)
-                gtk_style_context_remove_provider (gtk_widget_get_style_context (widget),
-                                                   GTK_STYLE_PROVIDER (properties));
-
-        g_object_set_data (G_OBJECT (widget), "panel-applet-style-props", NULL);
-}
-
 static void
 panel_applet_update_background_for_widget (GtkWidget       *widget,
 					   cairo_pattern_t *pattern)
 {
-        GtkStyleProperties *properties;
+	GdkWindow *window;
 
-        gtk_widget_reset_style (widget);
+	window = gtk_widget_get_window (widget);
 
-        if (!pattern) {
-                _panel_applet_reset_widget_style_properties (widget);
-                return;
-        }
+	if (!window)
+		return;
 
-        properties = _panel_applet_get_widget_style_properties (widget, TRUE);
-
-        switch (cairo_pattern_get_type (pattern)) {
-        case CAIRO_PATTERN_TYPE_SOLID: {
-                GdkRGBA color;
-
-                cairo_pattern_get_rgba (pattern, &color.red, &color.green, &color.blue, &color.alpha);
-                gtk_style_properties_set (properties, GTK_STATE_FLAG_NORMAL,
-                                          "background-color", &color,
-                                          "background-image", NULL,
-                                          NULL);
-        }
-                break;
-        case CAIRO_PATTERN_TYPE_SURFACE:
-                gtk_style_properties_set (properties, GTK_STATE_FLAG_NORMAL,
-					  /* background-color can't be NULL,
-					   * but is ignored anyway */
-                                          "background-image", pattern,
-                                          NULL);
-                break;
-        default:
-                break;
-        }
-
-	/* Note: this actually replaces the old properties, since it's the same
-	 * pointer */
-        gtk_style_context_add_provider (gtk_widget_get_style_context (widget),
-                                        GTK_STYLE_PROVIDER (properties),
-                                        GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+	gdk_window_set_background_pattern (window,
+	                                   pattern);
 }
 
 static void
@@ -1870,6 +1579,9 @@ panel_applet_get_property (GObject    *object,
 	PanelApplet *applet = PANEL_APPLET (object);
 
 	switch (prop_id) {
+	case PROP_OUT_OF_PROCESS:
+		g_value_set_boolean (value, applet->priv->out_of_process);
+		break;
 	case PROP_ID:
 		g_value_set_string (value, applet->priv->id);
 		break;
@@ -1882,15 +1594,9 @@ panel_applet_get_property (GObject    *object,
 	case PROP_SETTINGS_PATH:
 		g_value_set_string (value, applet->priv->settings_path);
 		break;
-	case PROP_PREFS_KEY:
-		g_value_set_string (value, applet->priv->prefs_key);
-		break;
 	case PROP_ORIENT:
 		g_value_set_uint (value, applet->priv->orient);
 		break;
-	case PROP_SIZE:
-		g_value_set_uint (value, applet->priv->size);
-		break;
 	case PROP_BACKGROUND:
 		g_value_set_string (value, applet->priv->background);
 		break;
@@ -1928,6 +1634,9 @@ panel_applet_set_property (GObject      *object,
 	PanelApplet *applet = PANEL_APPLET (object);
 
 	switch (prop_id) {
+	case PROP_OUT_OF_PROCESS:
+		applet->priv->out_of_process = g_value_get_boolean (value);
+		break;
 	case PROP_ID:
 		applet->priv->id = g_value_dup_string (value);
 		break;
@@ -1946,15 +1655,9 @@ panel_applet_set_property (GObject      *object,
 	case PROP_SETTINGS_PATH:
 		panel_applet_set_settings_path (applet, g_value_get_string (value));
 		break;
-	case PROP_PREFS_KEY:
-		panel_applet_set_preferences_key (applet, g_value_get_string (value));
-		break;
 	case PROP_ORIENT:
 		panel_applet_set_orient (applet, g_value_get_uint (value));
 		break;
-	case PROP_SIZE:
-		panel_applet_set_size (applet, g_value_get_uint (value));
-		break;
 	case PROP_BACKGROUND:
 		panel_applet_set_background_string (applet, g_value_get_string (value));
 		break;
@@ -2040,35 +1743,25 @@ panel_applet_init (PanelApplet *applet)
 {
 	GtkStyleContext *context;
 
-	applet->priv = PANEL_APPLET_GET_PRIVATE (applet);
+	applet->priv = panel_applet_get_instance_private (applet);
 
 	applet->priv->flags  = PANEL_APPLET_FLAGS_NONE;
 	applet->priv->orient = PANEL_APPLET_ORIENT_UP;
-	applet->priv->size   = 24;
 
-	applet->priv->client = gconf_client_get_default ();
+	applet->priv->panel_action_group = g_simple_action_group_new ();
+	g_action_map_add_action_entries (G_ACTION_MAP (applet->priv->panel_action_group),
+	                                 menu_entries,
+	                                 G_N_ELEMENTS (menu_entries),
+	                                 applet);
 
-	applet->priv->panel_action_group = gtk_action_group_new ("PanelActions");
-	gtk_action_group_set_translation_domain (applet->priv->panel_action_group, GETTEXT_PACKAGE);
-	gtk_action_group_add_actions (applet->priv->panel_action_group,
-				      menu_entries,
-				      G_N_ELEMENTS (menu_entries),
-				      applet);
+	applet->priv->builder = gtk_builder_new ();
 
-	applet->priv->ui_manager = gtk_ui_manager_new ();
-	gtk_ui_manager_insert_action_group (applet->priv->ui_manager,
-					    applet->priv->panel_action_group, 1);
-	gtk_ui_manager_add_ui_from_string (applet->priv->ui_manager,
-					   panel_menu_ui, -1, NULL);
+	gchar *xml = g_strdup_printf (panel_menu_ui, N_("_Remove From Panel"), N_("_Move"));
+	gtk_builder_add_from_string (applet->priv->builder, xml, -1, NULL);
+	g_free (xml);
 
-
-	panel_applet_bindings_init (applet->priv->client);
-
-
-	applet->priv->plug = gtk_plug_new (0);
-	g_signal_connect_swapped (G_OBJECT (applet->priv->plug), "embedded",
-				  G_CALLBACK (panel_applet_setup),
-				  applet);
+	gtk_widget_insert_action_group (GTK_WIDGET (applet), "libpanel-applet",
+	                                G_ACTION_GROUP (applet->priv->panel_action_group));
 
 	gtk_widget_set_events (GTK_WIDGET (applet),
 			       GDK_BUTTON_PRESS_MASK |
@@ -2076,8 +1769,33 @@ panel_applet_init (PanelApplet *applet)
 
 	context = gtk_widget_get_style_context (GTK_WIDGET (applet));
 	gtk_style_context_add_class (context, GTK_STYLE_CLASS_HORIZONTAL);
+}
+
+static GObject *
+panel_applet_constructor (GType                  type,
+                          guint                  n_construct_properties,
+                          GObjectConstructParam *construct_properties)
+{
+	GObject     *object;
+	PanelApplet *applet;
+
+	object = G_OBJECT_CLASS (panel_applet_parent_class)->constructor (type,
+	                                                                  n_construct_properties,
+	                                                                  construct_properties);
+	applet = PANEL_APPLET (object);
+
+	if (!applet->priv->out_of_process)
+		return object;
+
+	applet->priv->plug = gtk_plug_new (0);
+
+	g_signal_connect_swapped (G_OBJECT (applet->priv->plug), "embedded",
+		                      G_CALLBACK (panel_applet_setup),
+		                      applet);
 
 	gtk_container_add (GTK_CONTAINER (applet->priv->plug), GTK_WIDGET (applet));
+
+	return object;
 }
 
 static void
@@ -2102,6 +1820,7 @@ panel_applet_class_init (PanelAppletClass *klass)
 
 	gobject_class->get_property = panel_applet_get_property;
 	gobject_class->set_property = panel_applet_set_property;
+	gobject_class->constructor = panel_applet_constructor;
 	gobject_class->constructed = panel_applet_constructed;
         gobject_class->finalize = panel_applet_finalize;
 
@@ -2118,8 +1837,19 @@ panel_applet_class_init (PanelAppletClass *klass)
 	widget_class->focus = panel_applet_focus;
 	widget_class->realize = panel_applet_realize;
 
-	g_type_class_add_private (klass, sizeof (PanelAppletPrivate));
-
+	/**
+	 * PanelApplet:out-of-process: (skip)
+	 *
+	 * Implementation detail.
+	 **/
+	g_object_class_install_property (gobject_class,
+	                                 PROP_OUT_OF_PROCESS,
+	                                 g_param_spec_boolean ("out-of-process",
+	                                                       "out-of-process",
+	                                                       "out-of-process",
+	                                                       TRUE,
+	                                                       G_PARAM_CONSTRUCT_ONLY |
+	                                                       G_PARAM_READWRITE));
 	/**
 	 * PanelApplet:id: (skip)
 	 *
@@ -2172,22 +1902,6 @@ panel_applet_class_init (PanelAppletClass *klass)
 							      "GSettings path to per-instance settings",
 							      NULL,
 							      G_PARAM_READWRITE));
-	/**
-	 * PanelApplet:prefs-key:
-	 *
-	 * The GConf path to the per-instance settings of the applet.
-	 *
-	 * This property gets set when the applet gets embedded.
-	 *
-	 * Deprecated: 3.0: Use #GSettings to store per-instance settings.
-	 **/
-	g_object_class_install_property (gobject_class,
-					 PROP_PREFS_KEY,
-					 g_param_spec_string ("prefs-key",
-							      "PrefsKey",
-							      "GConf Preferences Key",
-							      NULL,
-							      G_PARAM_READWRITE));
 	/**
 	 * PanelApplet:orient:
 	 *
@@ -2205,25 +1919,6 @@ panel_applet_class_init (PanelAppletClass *klass)
 							    PANEL_APPLET_ORIENT_LAST,
 							    PANEL_APPLET_ORIENT_UP,
 							    G_PARAM_READWRITE));
-	/**
-	 * PanelApplet:size:
-	 *
-	 * The size of the panel the applet is on. For a horizontal panel, the
-	 * size if the height of the panel; for a vertical panel, the size is
-	 * the width of the panel.
-	 *
-	 * This property gets set when the applet gets embedded, and can change
-	 * when the panel size changes.
-         *
-	 * Deprecated: 3.0: Use the allocation of @applet instead.
-	 **/
-	g_object_class_install_property (gobject_class,
-					 PROP_SIZE,
-					 g_param_spec_uint ("size",
-							    "Size",
-							    "Panel Applet Size",
-							    0, G_MAXUINT, 0,
-							    G_PARAM_READWRITE));
 	/**
 	 * PanelApplet:background: (skip)
 	 *
@@ -2294,27 +1989,6 @@ panel_applet_class_init (PanelAppletClass *klass)
 			      1,
 			      G_TYPE_UINT);
 
-        /**
-         * PanelApplet::change-size:
-         * @applet: the #PanelApplet which emitted the signal.
-         * @size: the new size of the panel @applet is on.
-         *
-         * Emitted when the size of the panel @applet is on has changed.
-	 *
-	 * Deprecated: 3.0: Use the #GtkWidget::size-allocate signal instead.
-         **/
-	panel_applet_signals [CHANGE_SIZE] =
-                g_signal_new ("change_size",
-                              G_TYPE_FROM_CLASS (klass),
-                              G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (PanelAppletClass, change_size),
-                              NULL,
-			      NULL,
-                              g_cclosure_marshal_VOID__INT,
-                              G_TYPE_NONE,
-			      1,
-			      G_TYPE_INT);
-
         /**
          * PanelApplet::change-background:
          * @applet: the #PanelApplet which emitted the signal.
@@ -2407,13 +2081,8 @@ get_property_cb (GDBusConnection *connection,
 	if (g_strcmp0 (property_name, "SettingsPath") == 0) {
 		retval = g_variant_new_string (applet->priv->settings_path ?
 					       applet->priv->settings_path : "");
-	} else if (g_strcmp0 (property_name, "PrefsKey") == 0) {
-		retval = g_variant_new_string (applet->priv->prefs_key ?
-					       applet->priv->prefs_key : "");
 	} else if (g_strcmp0 (property_name, "Orient") == 0) {
 		retval = g_variant_new_uint32 (applet->priv->orient);
-	} else if (g_strcmp0 (property_name, "Size") == 0) {
-		retval = g_variant_new_uint32 (applet->priv->size);
 	} else if (g_strcmp0 (property_name, "Background") == 0) {
 		retval = g_variant_new_string (applet->priv->background ?
 					       applet->priv->background : "");
@@ -2450,12 +2119,8 @@ set_property_cb (GDBusConnection *connection,
 
 	if (g_strcmp0 (property_name, "SettingsPath") == 0) {
 		panel_applet_set_settings_path (applet, g_variant_get_string (value, NULL));
-	} else if (g_strcmp0 (property_name, "PrefsKey") == 0) {
-		panel_applet_set_preferences_key (applet, g_variant_get_string (value, NULL));
 	} else if (g_strcmp0 (property_name, "Orient") == 0) {
 		panel_applet_set_orient (applet, g_variant_get_uint32 (value));
-	} else if (g_strcmp0 (property_name, "Size") == 0) {
-		panel_applet_set_size (applet, g_variant_get_uint32 (value));
 	} else if (g_strcmp0 (property_name, "Background") == 0) {
 		panel_applet_set_background_string (applet, g_variant_get_string (value, NULL));
 	} else if (g_strcmp0 (property_name, "Flags") == 0) {
@@ -2485,9 +2150,7 @@ static const gchar introspection_xml[] =
 	      "<arg name='time' type='u' direction='in'/>"
 	    "</method>"
 	    "<property name='SettingsPath' type='s' access='readwrite'/>"
-	    "<property name='PrefsKey' type='s' access='readwrite'/>"
 	    "<property name='Orient' type='u' access='readwrite' />"
-	    "<property name='Size' type='u' access='readwrite'/>"
 	    "<property name='Background' type='s' access='readwrite'/>"
 	    "<property name='Flags' type='u' access='readwrite'/>"
 	    "<property name='SizeHints' type='ai' access='readwrite'/>"
@@ -2607,7 +2270,7 @@ _panel_applet_factory_main_internal (const gchar               *factory_id,
 		_panel_applet_setup_x_error_handler ();
 
 	closure = g_cclosure_new (G_CALLBACK (callback), user_data, NULL);
-	factory = panel_applet_factory_new (factory_id, applet_type, closure);
+	factory = panel_applet_factory_new (factory_id, out_process, applet_type, closure);
 	g_closure_unref (closure);
 
 	if (panel_applet_factory_register_service (factory)) {
@@ -2724,6 +2387,9 @@ guint32
 panel_applet_get_xid (PanelApplet *applet,
 		      GdkScreen   *screen)
 {
+	if (applet->priv->out_of_process == FALSE)
+		return 0;
+
 	gtk_window_set_screen (GTK_WINDOW (applet->priv->plug), screen);
 	gtk_widget_show (applet->priv->plug);
 
@@ -2735,3 +2401,19 @@ panel_applet_get_object_path (PanelApplet *applet)
 {
 	return applet->priv->object_path;
 }
+
+G_MODULE_EXPORT GtkWidget *
+panel_applet_get_applet_widget (const gchar *factory_id,
+                                guint        uid)
+{
+	GtkWidget *widget;
+
+	widget = panel_applet_factory_get_applet_widget (factory_id, uid);
+	if (!widget) {
+		return NULL;
+	}
+
+	panel_applet_setup (PANEL_APPLET (widget));
+
+	return widget;
+}
